@@ -139,11 +139,14 @@ class LocalClassifierPerLevel(BaseEstimator, HierarchicalClassifier):
         for level, classifier in enumerate(self.local_classifiers_):
             self.logger_.info(f"Predicting level {level}")
             if level == 0:
-                y[:, level] = classifier.predict(X)
+                y[:, level] = classifier.predict(X).flatten()
             else:
                 all_probabilities = classifier.predict_proba(X)
                 successors = np.array(
-                    [list(self.hierarchy_.successors(node)) for node in y[:, level - 1]],
+                    [
+                        list(self.hierarchy_.successors(node))
+                        for node in y[:, level - 1]
+                    ],
                     dtype=object,
                 )
                 classes_masks = np.array(
@@ -159,7 +162,10 @@ class LocalClassifierPerLevel(BaseEstimator, HierarchicalClassifier):
                     ],
                     dtype=object,
                 )
-                highest_probabilities = [np.argmax(probabilities[i], axis=0) for i in range(len(probabilities))]
+                highest_probabilities = [
+                    np.argmax(probabilities[i], axis=0)
+                    for i in range(len(probabilities))
+                ]
                 classes = np.array(
                     [
                         classifier.classes_[classes_masks[i]]
