@@ -3,18 +3,17 @@ Local classifier per node approach.
 
 Numeric and string output labels are both handled.
 """
-import logging
 from copy import deepcopy
 
 import networkx as nx
 import numpy as np
 import ray
 from sklearn.base import BaseEstimator
-from sklearn.linear_model import LogisticRegression
-from sklearn.utils.validation import check_X_y, check_array, check_is_fitted
+from sklearn.utils.validation import check_array, check_is_fitted
 
 from hiclass import BinaryPolicy
 from hiclass.ConstantClassifier import ConstantClassifier
+from hiclass.HierarchicalClassifier import HierarchicalClassifier
 
 
 @ray.remote
@@ -28,7 +27,7 @@ def _parallel_fit(lcpn, node):
     return classifier
 
 
-class LocalClassifierPerNode(BaseEstimator):
+class LocalClassifierPerNode(BaseEstimator, HierarchicalClassifier):
     """
     Assign local classifiers to each node of the graph, except the root node.
 
@@ -67,12 +66,15 @@ class LocalClassifierPerNode(BaseEstimator):
         n_jobs : int, default=1
             The number of jobs to run in parallel. Only :code:`fit` is parallelized.
         """
-        self.local_classifier = local_classifier
+        super().__init__(
+            local_classifier=local_classifier,
+            verbose=verbose,
+            edge_list=edge_list,
+            replace_classifiers=replace_classifiers,
+            n_jobs=n_jobs,
+            classifier_abbreviation="LCPN",
+        )
         self.binary_policy = binary_policy
-        self.verbose = verbose
-        self.edge_list = edge_list
-        self.replace_classifiers = replace_classifiers
-        self.n_jobs = n_jobs
 
     def fit(self, X, y):
         """
@@ -92,44 +94,14 @@ class LocalClassifierPerNode(BaseEstimator):
         self : object
             Fitted estimator.
         """
-        # Check that X and y have correct shape
-        # and convert them to np.ndarray if need be
-        self.X_, self.y_ = check_X_y(X, y, multi_output=True, accept_sparse="csr")
-
-        # Create and configure logger
-        self._create_logger()
-
-        # Avoids creating more columns in prediction if edges are a->b and b->c,
-        # which would generate the prediction a->b->c
-        self._disambiguate()
-
-        # Create DAG from self.y_ and store to self.hierarchy_
-        self._create_digraph()
-
-        # If user passes edge_list, then export
-        # DAG to CSV file to visualize with Gephi
-        self._export_digraph()
-
-        # Assert that graph is directed acyclic
-        self._assert_digraph_is_dag()
-
-        # If y is 1D, convert to 2D for binary policies
-        self._convert_1d_y_to_2d()
+        # Execute common methods necessary before fitting
+        super()._pre_fit(X, y)
 
         # Initialize policy
         self._initialize_binary_policy()
 
-        # Detect root(s) and add artificial root to DAG
-        self._add_artificial_root()
-
-        # Initialize local classifiers in DAG
-        self._initialize_local_classifiers()
-
         # Fit local classifiers in DAG
-        if self.n_jobs > 1:
-            self._fit_digraph_parallel()
-        else:
-            self._fit_digraph()
+        super().fit(X, y)
 
         # TODO: Store the classes seen during fit
 
@@ -139,11 +111,6 @@ class LocalClassifierPerNode(BaseEstimator):
 
         # TODO: Add support to empty labels in some levels
 
-        # TODO: Parallelize fit
-
-        # Delete unnecessary variables
-        self._clean_up()
-
         # Return the classifier
         return self
 
@@ -203,8 +170,6 @@ class LocalClassifierPerNode(BaseEstimator):
                     self.hierarchy_, self.root_, predecessor
                 )
                 prediction = np.array(prediction)
-                if prediction.ndim == 2 and prediction.shape[1] == 1:
-                    prediction = prediction.flatten()
                 y[mask, level] = prediction
 
         # Convert back to 1D if there is only 1 column to pass all sklearn's checks
@@ -219,100 +184,6 @@ class LocalClassifierPerNode(BaseEstimator):
 
         return y
 
-    def _create_logger(self):
-        # Create logger
-        self.logger_ = logging.getLogger("LCPN")
-        self.logger_.setLevel(self.verbose)
-
-        # Create console handler and set verbose level
-        ch = logging.StreamHandler()
-        ch.setLevel(self.verbose)
-
-        # Create formatter
-        formatter = logging.Formatter(
-            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
-        )
-
-        # Add formatter to ch
-        ch.setFormatter(formatter)
-
-        # Add ch to logger
-        self.logger_.addHandler(ch)
-
-    def _disambiguate(self):
-        self.separator_ = "::HiClass::Separator::"
-        if self.y_.ndim == 2:
-            new_y = []
-            for i in range(self.y_.shape[0]):
-                row = [self.y_[i, 0]]
-                for j in range(1, self.y_.shape[1]):
-                    row.append(str(row[-1]) + self.separator_ + str(self.y_[i, j]))
-                new_y.append(np.asarray(row, dtype=np.str_))
-            self.y_ = np.array(new_y)
-
-    def _create_digraph(self):
-        # Create DiGraph
-        self.hierarchy_ = nx.DiGraph()
-
-        # Save dtype of y_
-        self.dtype_ = self.y_.dtype
-
-        # 1D disguised as 2D
-        if self.y_.ndim == 2 and self.y_.shape[1] == 1:
-            self.logger_.info("Converting y to 1D")
-            self.y_ = self.y_.flatten()
-
-        # Check dimension of labels
-        if self.y_.ndim == 2:
-            # 2D labels
-            # Create max_levels variable
-            self.max_levels_ = self.y_.shape[1]
-            rows, columns = self.y_.shape
-            self.logger_.info(f"Creating digraph from {rows} 2D labels")
-            for row in range(rows):
-                for column in range(columns - 1):
-                    self.hierarchy_.add_edge(
-                        self.y_[row, column], self.y_[row, column + 1]
-                    )
-
-        elif self.y_.ndim == 1:
-            # 1D labels
-            # Create max_levels_ variable
-            self.max_levels_ = 1
-            self.logger_.info(f"Creating digraph from {self.y_.size} 1D labels")
-            for label in self.y_:
-                self.hierarchy_.add_node(label)
-
-        else:
-            # Unsuported dimension
-            self.logger_.error(f"y with {self.y_.ndim} dimensions detected")
-            raise ValueError(
-                f"Creating graph from y with {self.y_.ndim} dimensions is not supported"
-            )
-
-    def _convert_1d_y_to_2d(self):
-        # This conversion is necessary for the binary policies
-        if self.y_.ndim == 1:
-            self.y_ = np.reshape(self.y_, (-1, 1))
-
-    def _export_digraph(self):
-        # Check if edge_list is set
-        if self.edge_list:
-            # Add quotes to all nodes in case the text has commas
-            mapping = {}
-            for node in self.hierarchy_:
-                mapping[node] = '"{}"'.format(node.split(self.separator_)[-1])
-            hierarchy = nx.relabel_nodes(self.hierarchy_, mapping, copy=True)
-            # Export DAG to CSV file
-            self.logger_.info(f"Writing edge list to file {self.edge_list}")
-            nx.write_edgelist(hierarchy, self.edge_list, delimiter=",")
-
-    def _assert_digraph_is_dag(self):
-        # Assert that graph is directed acyclic
-        if not nx.is_directed_acyclic_graph(self.hierarchy_):
-            self.logger_.error("Cycle detected in graph")
-            raise ValueError("Graph is not directed acyclic")
-
     def _initialize_binary_policy(self):
         if isinstance(self.binary_policy, str):
             self.logger_.info(f"Initializing {self.binary_policy} binary policy")
@@ -332,27 +203,9 @@ class LocalClassifierPerNode(BaseEstimator):
                 f"Binary policy type must str, not {type(self.binary_policy)}."
             )
 
-    def _add_artificial_root(self):
-        # Detect root(s)
-        roots = [
-            node for node, in_degree in self.hierarchy_.in_degree() if in_degree == 0
-        ]
-        self.logger_.info(f"Detected {len(roots)} roots")
-
-        # Add artificial root as predecessor to root(s) detected
-        self.root_ = "hiclass::root"
-        for old_root in roots:
-            self.hierarchy_.add_edge(self.root_, old_root)
-
     def _initialize_local_classifiers(self):
-        # Create a deep copy of the local classifier specified
-        # for each node in the hierarchy and save to attribute "classifier"
-        self.logger_.info("Initializing local classifiers")
+        super()._initialize_local_classifiers()
         local_classifiers = {}
-        if self.local_classifier is None:
-            self.local_classifier_ = LogisticRegression()
-        else:
-            self.local_classifier_ = self.local_classifier
         for node in self.hierarchy_.nodes:
             # Skip only root node
             if node != self.root_:
@@ -396,7 +249,5 @@ class LocalClassifierPerNode(BaseEstimator):
             classifier.fit(X, y)
 
     def _clean_up(self):
-        self.logger_.info("Cleaning up variables that can take a lot of disk space")
-        del self.X_
-        del self.y_
+        super()._clean_up()
         del self.binary_policy_
