@@ -4,12 +4,14 @@ from sklearn.utils import check_array
 
 
 def precision(y_true: np.ndarray, y_pred: np.ndarray):
-    """
+    r"""
     Compute precision score for hierarchical classification.
 
-    hP = sum(len(S intersection T)) / sum(len(S)),
-    where S is the set consisting of the most specific class(es) predicted for a test example and all respective ancestors
-    and T is the set consisting of the true most specific class(es) for a test example and all respective ancestors.
+    :math:`hP = \displaystyle{\frac{\sum_{i}| \alpha_i \cap \beta_i |}{\sum_{i}| \alpha_i |}}`,
+    where :math:`\alpha_i` is the set consisting of the most specific classes predicted
+    for test example :math:`i` and all their ancestor classes, while :math:`\beta_i` is the
+    set containing the true most specific classes of test example :math:`i` and all
+    their ancestors, with summations computed over all test examples.
 
     Parameters
     ----------
@@ -39,12 +41,14 @@ def precision(y_true: np.ndarray, y_pred: np.ndarray):
 
 
 def recall(y_true: np.ndarray, y_pred: np.ndarray):
-    """
+    r"""
     Compute recall score for hierarchical classification.
 
-    hR = sum(len(S intersection T)) / sum(len(T)),
-    where S is the set consisting of the most specific class(es) predicted for a test example and all respective ancestors
-    and T is the set consisting of the true most specific class(es) for a test example and all respective ancestors.
+    :math:`\displaystyle{hR = \frac{\sum_i|\alpha_i \cap \beta_i|}{\sum_i|\beta_i|}}`,
+    where :math:`\alpha_i` is the set consisting of the most specific classes predicted
+    for test example :math:`i` and all their ancestor classes, while :math:`\beta_i` is the
+    set containing the true most specific classes of test example :math:`i` and all
+    their ancestors, with summations computed over all test examples.
 
     Parameters
     ----------
@@ -74,11 +78,11 @@ def recall(y_true: np.ndarray, y_pred: np.ndarray):
 
 
 def f1(y_true: np.ndarray, y_pred: np.ndarray):
-    """
+    r"""
     Compute f1 score for hierarchical classification.
 
-    hF = 2 * hP * hR / (hP + hR),
-    where hP is the hierarchical precision and hR is the hierarchical recall.
+    :math:`\displaystyle{hF = \frac{2 \times hP \times hR}{hP + hR}}`,
+    where :math:`hP` is the hierarchical precision and :math:`hR` is the hierarchical recall.
 
     Parameters
     ----------
